"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.metaReducer = exports.DEFAULT_METAS = exports.METAS_PICK_METHOD = exports.DEFAULT_META = exports.USED_FLAG = void 0;
var tslib_1 = require("tslib");
var BaseReducer_1 = require("./BaseReducer");
var actionType_1 = require("../utils/actionType");
var StoreHelper_1 = require("../stores/StoreHelper");
exports.USED_FLAG = 'meta-used';
exports.DEFAULT_META = {
    actionType: '',
    loading: false,
};
exports.METAS_PICK_METHOD = {
    pick: function (payload) {
        return this[payload] || exports.DEFAULT_META;
    },
};
// @ts-ignore
exports.DEFAULT_METAS = tslib_1.__assign({}, exports.METAS_PICK_METHOD);
var REDUCER_NAME = '__metas__';
var MetaReducer = /** @class */ (function (_super) {
    tslib_1.__extends(MetaReducer, _super);
    function MetaReducer() {
        var _this = _super.call(this, REDUCER_NAME, {}, []) || this;
        // All metas will be stored here before user need them.
        _this.stash = {};
        return _this;
    }
    MetaReducer.prototype.getData = function (name) {
        return StoreHelper_1.storeHelper.getState()[REDUCER_NAME][name];
    };
    MetaReducer.prototype.getMeta = function (name) {
        var stash = this.stash[name];
        if (!this.getData(name) && stash && stash !== exports.USED_FLAG) {
            var action = {
                type: actionType_1.ACTION_TYPE_META_RESTORE,
                payload: {
                    key: name,
                    value: stash,
                },
            };
            StoreHelper_1.storeHelper.dispatch(action);
        }
        if (stash !== exports.USED_FLAG) {
            this.stash[name] = exports.USED_FLAG;
        }
        return this.getData(name);
    };
    MetaReducer.prototype.isRestore = function (action) {
        return action.type === actionType_1.ACTION_TYPE_META_RESTORE;
    };
    MetaReducer.prototype.reducer = function (state, action) {
        var _a, _b, _c, _d;
        if (state === undefined) {
            return this.initData;
        }
        if (this.isRestore(action)) {
            return tslib_1.__assign(tslib_1.__assign({}, state), (_a = {}, _a[action.payload.key] = action.payload.value, _a));
        }
        var metaKey = action.metaKey;
        var actionName = action.metaActionName;
        if (!actionName || metaKey === undefined || metaKey === false) {
            return state;
        }
        var meta = {
            actionType: action.type,
            loading: action.loading,
            message: action.message,
            httpStatus: action.httpStatus,
            businessCode: action.businessCode,
        };
        var used = this.stash[actionName] === exports.USED_FLAG;
        if (metaKey === true) {
            if (used) {
                return tslib_1.__assign(tslib_1.__assign({}, state), (_b = {}, _b[actionName] = meta, _b));
            }
            this.stash[actionName] = meta;
        }
        else {
            if (used) {
                return tslib_1.__assign(tslib_1.__assign({}, state), (_c = {}, _c[actionName] = tslib_1.__assign(tslib_1.__assign(tslib_1.__assign({}, state[actionName]), (_d = {}, _d[metaKey] = meta, _d)), exports.METAS_PICK_METHOD), _c));
            }
            // @ts-ignore
            this.stash[actionName] = this.stash[actionName] || tslib_1.__assign({}, exports.METAS_PICK_METHOD);
            this.stash[actionName][metaKey] = meta;
        }
        return state;
    };
    return MetaReducer;
}(BaseReducer_1.BaseReducer));
exports.metaReducer = new MetaReducer();
StoreHelper_1.storeHelper.appendReducers(exports.metaReducer.createReducer());
